function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import React, { useContext, useMemo } from 'react';
import RcFieldForm from 'rc-field-form';
import { Form as NativeBaseForm, View, Text, Row, Col, Grid } from 'native-base';
import CommonAdapters from './CommonAdapters';
const FormXContext = React.createContext({
  form: undefined
});

const SchemaFormItem = props => {
  const {
    field,
    adapters
  } = props;
  const fType = useMemo(() => {
    const fType = field.type;

    if (typeof fType === 'string') {
      return fType;
    }

    if (fType.hasOwnProperty('object')) {
      return 'object';
    }

    return 'array';
  }, [field.type]);
  const namePath = useMemo(() => {
    var _props$fieldData;

    const fName = (_props$fieldData = props.fieldData) === null || _props$fieldData === void 0 ? void 0 : _props$fieldData.name;

    if (fName === undefined) {
      return [field.id];
    }

    return [fName, field.id];
  }, [field.id, props]);
  const options = useMemo(() => {
    var _field$options;

    if (!((_field$options = field.options) === null || _field$options === void 0 ? void 0 : _field$options.rules) || field.options.rules.findIndex(v => v.hasOwnProperty('validator') && typeof v.validator === 'string') === -1) {
      return field.options;
    }

    const opts = _objectSpread({}, field.options);

    opts.rules = opts.rules.map(rule => {
      if (!rule.hasOwnProperty('validator') || typeof rule.validator !== 'string') {
        return rule;
      }

      const adapt = props.validationAdapters[rule.validator];

      if (adapt === undefined) {
        return rule;
      }

      const message = rule.message ? rule.message : adapt.message;
      return form => _objectSpread(_objectSpread({}, rule), {}, {
        message,
        validator: async (r, value) => adapt.validator(r, value, form)
      });
    });
    return opts;
  }, [field.options, props.validationAdapters]);
  const form = useContext(FormXContext).form;
  const adapter = useMemo(() => adapters[fType] ? adapters[fType] : adapters.string, [adapters, fType]); // const toSentenceCase = (val: string) => {
  //   const temp = val.replace(/([A-Z])/g, ' $1').replace(/[']/g, '');
  //   return temp.charAt(0).toUpperCase() + temp.slice(1);
  // };

  const repaceNamePath = val => {
    return val.replace(/'(.*?)'/g, '*');
  };

  return /*#__PURE__*/React.createElement(RcFieldForm.Field, _extends({}, options, {
    label: field.label,
    name: namePath
  }), ps => {
    var _field$props;

    return /*#__PURE__*/React.createElement(View, {
      style: {
        flex: 1
      }
    }, adapter.render(field, _objectSpread(_objectSpread(_objectSpread(_objectSpread({}, field.props), ps), field), {}, {
      validationInfo: form.getFieldError(namePath).map(item => repaceNamePath(item)),
      type: field === null || field === void 0 ? void 0 : (_field$props = field.props) === null || _field$props === void 0 ? void 0 : _field$props.type
    })), (field === null || field === void 0 ? void 0 : field.label) === undefined && form.getFieldError(namePath).length > 0 && /*#__PURE__*/React.createElement(Text, {
      style: {
        color: '#f39c12',
        marginLeft: 5
      }
    }, form.getFieldError(namePath).map(item => repaceNamePath(item))));
  });
};

const SchemaFormItemGroup = props => {
  const {
    field,
    adapters
  } = props; // console.log('from group ', props);

  return /*#__PURE__*/React.createElement(SchemaFormItem, {
    field: field,
    adapters: adapters,
    fieldData: props.fieldData,
    validationAdapters: props.validationAdapters,
    _adapters: props._adapters,
    _validationAdapters: props._validationAdapters,
    parentPath: props.parentPath
  });
};

const SchemaFormFields = props => {
  const {
    schema,
    adapters,
    validationAdapters
  } = props;
  const form = useContext(FormXContext);
  const fields = useMemo(() => {
    const f = [];
    let group = [];

    for (const field of schema) {
      if (group.length > 0) {
        f.push(group);
        group = [];
      }

      f.push(field);
    }

    if (group.length > 0) {
      f.push(group);
    }

    return f;
  }, [form.form, form.value, schema]);
  return /*#__PURE__*/React.createElement(React.Fragment, null, fields.map((field, index) => /*#__PURE__*/React.createElement(SchemaFormItemGroup, {
    field: field,
    adapters: adapters,
    key: index,
    fieldData: props.fieldData,
    validationAdapters: validationAdapters,
    _adapters: props._adapters,
    _validationAdapters: props._validationAdapters,
    parentPath: props.parentPath
  })));
};

const SchemaFormItems = props => {
  const {
    schema
  } = props;
  const adapters = useMemo(() => {
    const ads = {};
    CommonAdapters.all().forEach(value => ads[value.name] = value);

    if (props.adapters) {
      props.adapters.forEach(value => ads[value.name] = value);
    }

    return ads;
  }, [props.adapters]);
  const validationAdapters = useMemo(() => {
    const ads = {};
    CommonAdapters.allValidators().forEach(value => ads[value.name] = value);

    if (props.validationAdapters) {
      props.validationAdapters.forEach(value => ads[value.name] = value);
    }

    return ads;
  }, [props.validationAdapters]);
  const [sch, typ] = useMemo(() => {
    const sx = Array.isArray(schema) ? schema : [schema];
    const ty = sx.length > 0 && sx[0].hasOwnProperty('fields') ? 'columns' : 'fields';
    return [sx, ty];
  }, []);

  if (sch.length === 0) {
    return /*#__PURE__*/React.createElement(React.Fragment, null);
  }

  if (typ === 'fields') {
    return /*#__PURE__*/React.createElement(SchemaFormFields, {
      schema: sch,
      adapters: adapters,
      validationAdapters: validationAdapters,
      fieldData: props.fieldData,
      _adapters: props.adapters,
      _validationAdapters: props.validationAdapters,
      parentPath: props.parentPath ? props.parentPath : []
    });
  }

  return /*#__PURE__*/React.createElement(Grid, {
    style: {
      flexDirection: 'column'
    }
  }, sch.map((value, index) => {
    if (value.layout === 'vertical') {
      return /*#__PURE__*/React.createElement(Col, {
        key: index,
        size: 24,
        style: {
          flex: 1,
          width: '100%'
        }
      }, /*#__PURE__*/React.createElement(SchemaFormFields, {
        schema: value.fields,
        adapters: adapters,
        validationAdapters: validationAdapters,
        fieldData: props.fieldData,
        parentPath: props.parentPath ? props.parentPath : []
      }));
    }

    return /*#__PURE__*/React.createElement(Row, _extends({}, value.layout, {
      key: index,
      style: {
        flex: 1,
        width: '100%'
      }
    }), /*#__PURE__*/React.createElement(SchemaFormFields, {
      schema: value.fields,
      adapters: adapters,
      validationAdapters: validationAdapters,
      fieldData: props.fieldData,
      parentPath: props.parentPath ? props.parentPath : []
    }));
  }));
};

export default class Form extends React.Component {
  constructor(...args) {
    super(...args);

    _defineProperty(this, "state", {});

    _defineProperty(this, "onValuesChange", (changed, all) => {
      this.setState({
        value: all
      });

      if (this.props.onValuesChange) {
        this.props.onValuesChange(changed, all);
      }
    });
  }

  render() {
    return /*#__PURE__*/React.createElement(FormXContext.Provider, {
      value: {
        value: this.state.value,
        form: this.props.form
      }
    }, /*#__PURE__*/React.createElement(RcFieldForm, _extends({}, this.props, {
      onValuesChange: this.onValuesChange,
      component: NativeBaseForm
    }), this.props.children));
  }

}

_defineProperty(Form, "Item", RcFieldForm.Field);

_defineProperty(Form, "useForm", RcFieldForm.useForm);

_defineProperty(Form, "Provider", RcFieldForm.FormProvider);

_defineProperty(Form, "Items", SchemaFormItems);
//# sourceMappingURL=Form.js.map